import psycopg2
import psycopg2.extras

from dataclasses import dataclass


@dataclass
class PostgresAuthData:
    dbname: str
    user: str
    password: str
    host: str
    port: int


def make_dict_rows(rows: list) -> list:
    dict_list = []
    for row in rows:
        dict_list.append(dict(row))
    return dict_list


class DataBase:
    def __init__(self, pg_auth_data: PostgresAuthData):
        self.conn = psycopg2.connect(dbname=pg_auth_data.dbname, user=pg_auth_data.user, password=pg_auth_data.password,
                                     host=pg_auth_data.host, port=pg_auth_data.port)
        self.cursor = self.conn.cursor(cursor_factory=psycopg2.extras.DictCursor)

    def request(self, sql_request, args: tuple = None):
        try:
            if args:
                self.cursor.execute(sql_request, args)
                return make_dict_rows(self.cursor.fetchall())
            else:
                self.cursor.execute(sql_request)
                return make_dict_rows(self.cursor.fetchall())
        except psycopg2.ProgrammingError:
            return None

    def commit(self):
        self.conn.commit()



from .users import Users
from .messages import Messages
from tgusers.database.database import DataBase


class Tables:
    def __init__(self, db: DataBase):
        self.users      = Users("Users", db)
        self.messages   = Messages("Messages", db)


from tgusers.database.database import DataBase


class Table:
    def __init__(self, table_name, db: DataBase):
        self.table_name = table_name
        self.db = db

    def select_all_table_data(self):
        sql = """
                    SELECT *
                    FROM "%s";
        """ % (self.table_name, )
        return self.db.request(sql)



from random import randint
from dataclasses import dataclass
from tgusers.tables.tables import Tables
from tgusers.database.database import DataBase
"""
Room content type can be in range of ['audio', 'photo', 'voice', 'video', 'document', 'text', 'location', 'contact', 'sticker']
"""


@dataclass
class Room:
    name: str
    content_type: list
    roles: list
    function: any
    is_global: bool = False
    message_handler: bool = False


class RoomsErrors(Exception):
    def __init__(self, message):
        self.message = message


class RoomsAlerts:
    @staticmethod
    def access_denied(get_alerts):
        if get_alerts:
            return [False, "Access denied"]
        else:
            return False

    @staticmethod
    def room_not_found(get_alerts):
        if get_alerts:
            return [False, "Room not found"]
        else:
            return False

    @staticmethod
    def successfully(get_alerts):
        if get_alerts:
            return [True, "Successfully"]
        else:
            return True


def valid_check(string: str):
    valid_chars = "abcdefghijklmnopqrstuvwxyz1234567890_-"
    for char in string:
        if char not in valid_chars:
            return False
    return True


class Rooms:
    def __init__(self, tables: Tables, db: DataBase, get_alerts: bool = False):
        self.db = db
        self.rooms = {}
        self.tables = tables
        self.get_alerts = get_alerts

    def add_message_room(self, name: str = None, content_type: list = None, roles: list = None,
                         is_global: bool = False):
        if roles is None:
            roles = ["all"]
        if content_type is None:
            content_type = []
        if name is None:
            name = str(randint(100000, 999999))
        if not valid_check(name):
            raise RoomsErrors("Invalid room name")


        def decorator(room_func):
            if is_global:
                room = Room(name=None, content_type=content_type, roles=roles, function=room_func, is_global=True,
                            message_handler=True)
                self.rooms[name] = room
            else:
                room = Room(name=name, content_type=content_type, roles=roles, function=room_func, is_global=False,
                            message_handler=True)
                self.rooms[name] = room

        return decorator


    def add_role(self, role: str, users: list):
        for user in users:
            if not self.tables.users.check_user_for_registration(telegram_id=user):
                users.pop(user)
        for user in users:
            self.tables.users.set_user_role(role, telegram_id=user)
        self.db.commit()


    def user_go_to_room(self, message, room_name: str):
        print(self.tables.users.get_user_role(message))
        if self.tables.users.get_user_role(message) in self.rooms.get(room_name).roles or "all" in self.rooms.get(
                room_name).roles:
            if self.tables.users.set_room(message, self.rooms.keys(), room_name):
                return RoomsAlerts.successfully(self.get_alerts)
            else:
                return RoomsAlerts.room_not_found(self.get_alerts)
        else:
            return RoomsAlerts.access_denied(self.get_alerts)

    def go_to_one_of_the_rooms(self, message, rooms_dict: dict):
        get_room = rooms_dict.get(message.text)
        if get_room:
            return self.user_go_to_room(message, get_room)
        else:
            return False

    def show_rooms(self):
        print(self.rooms)



import tgusers

from dataclasses import dataclass


@dataclass
class User:
    telegram_id: int
    first_name: str
    last_name: str
    user_name: str
    phone: str
    language: str
    role: str
    room: str
    id: int = -1


class Users(tgusers.Table):
    def get_user_room(self, message):
        sql = """
                    SELECT room
                    FROM "Users"
                    WHERE telegram_id = %s;
        """
        return self.db.request(sql, (message.chat.id,))[0].get("room")

    def check_user_for_registration(self, message=None, telegram_id: int = None):
        if message:
            telegram_id = message.chat.id
        sql = """
                    SELECT id
                    FROM "Users"
                    WHERE telegram_id = %s;
                """
        if self.db.request(sql, (telegram_id, )):
            return True
        else:
            return False

    def set_user_role(self, role_name: str, message=None, telegram_id: int = None):
        if message:
            telegram_id = message.chat.id
        sql = """   UPDATE "Users" 
                    SET role = %s
                    WHERE telegram_id = %s;"""
        self.db.request(sql, (role_name, telegram_id))

    def get_user(self, message):
        sql = """
                    SELECT id, telegram_id, first_name, last_name, user_name, phone, language, role, room
                    FROM "Users"
                    WHERE telegram_id = %s;
                """
        response = self.db.request(sql, (message.chat.id,))[0]
        return User(**response)

    def get_user_role(self, message):
        sql = """
                    SELECT role
                    FROM "Users"
                    WHERE telegram_id = %s;
                """
        return self.db.request(sql, (message.chat.id, ))[0]["role"]


    def set_room(self, message, rooms: list, room_name: str):
        for room in rooms:
            if room == room_name:
                sql = """   UPDATE "Users" 
                            SET room = %s
                            WHERE telegram_id = %s;"""
                self.db.request(sql, (room_name, message.chat.id))
                self.db.commit()
                return True
        return False

    def add(self, user: User):
        sql = """   INSERT INTO "Users" (telegram_id, first_name, last_name, user_name, phone, language, role, room)
                    VALUES (%s, %s, %s, %s, %s, %s, %s, %s)
                    RETURNING id;
        """
        response = self.db.request(sql, (
            user.telegram_id, user.first_name, user.last_name, user.user_name, user.phone, user.language,
            user.role, user.room))
        self.db.commit()
        return response



from .database.database import PostgresAuthData
from .database.database import DataBase
from .class_models.table import Table
from .connect import Connect

__all__ = [
    "PostgresAuthData",
    "DataBase",
    "Table",
    "Connect"
]


import tgusers

from dataclasses import dataclass


@dataclass
class Message:
    user_id: int
    message: str
    message_id: int
    time: int


class Messages(tgusers.Table):
    def add(self, message: Message):
        sql = """   INSERT INTO "Messages" (user_id, message, message_id, time)
                    VALUES (%s, %s, %s, %s)
                    RETURNING id;
        """
        self.db.request(sql, (message.user_id, message.message, message.message_id, message.time))
        self.db.commit()



import telebot

from aiogram import Bot, Dispatcher, executor, types

from time import time
from tgusers.tables.users import User
from tgusers.tables.tables import Tables
from tgusers.rooms.room_class import Rooms
from tgusers.tables.messages import Message


class TelegramBot:
    def __init__(self, api_key: str, tables: Tables, rooms: Rooms, message_logging: bool):
        self.bot = Bot(api_key)
        self.disp = Dispatcher(self.bot)
        self.rooms = rooms
        self.tables = tables

        @self.disp.message_handler(content_types=['audio', 'photo', 'voice', 'video', 'document', 'text', 'location', 'contact', 'sticker'])
        async def handler(message: types.Message):
            if not self.tables.users.check_user_for_registration(message):
                user = User(telegram_id=message.chat.id, first_name=message.chat.first_name,
                            last_name=message.chat.last_name, user_name=message.chat.username,
                            phone="", language=message.from_user.language_code, role="user",
                            room="start")
                user.id = self.tables.users.add(user).get("id")
            else:
                user = self.tables.users.get_user(message)
            if message_logging:
                log_message = Message(user_id=user.id, message=message.text, message_id=message.message_id, time=round(time()))
                self.tables.messages.add(log_message)
                print(message.chat.username, ":", message.chat.id, " -> ", message.text, "[", message.content_type, "]")
            for room_name in self.rooms.rooms:
                room = self.rooms.rooms[room_name]
                if room.message_handler and message.content_type in room.content_type and (room.name == user.room or room.is_global):
                    await room.function(message)

    def polling(self):
        executor.start_polling(self.disp, skip_updates=False)



from tgusers.bot.bot_class import TelegramBot
from tgusers.database.database import DataBase, PostgresAuthData
from .rooms.room_class import Rooms, Room
from .tables.tables import Tables


class Connect:
    def __init__(self, bot_token: str, pgData: PostgresAuthData, message_logging: bool = False):
        self.data_base    = DataBase(pgData)
        self.tables       = Tables(self.data_base)
        self.rooms        = Rooms(self.tables, self.data_base)
        self.telegram_bot = TelegramBot(bot_token, self.tables, self.rooms, message_logging)

